@@ -1,15 +1,17 @@
 package commands
 
 import (
+	"bufio"
+	"fmt"
+	"io"
 	"path/filepath"
 	"strings"
 
 	"github.com/git-lfs/git-lfs/errors"
 	"github.com/git-lfs/git-lfs/git"
 	"github.com/git-lfs/git-lfs/git/githistory"
-	"github.com/git-lfs/git-lfs/git/githistory/log"
-	"github.com/git-lfs/git-lfs/git/odb"
-	"github.com/git-lfs/git-lfs/localstorage"
+	"github.com/git-lfs/git-lfs/tasklog"
+	"github.com/git-lfs/gitobj"
 	"github.com/spf13/cobra"
 )
 
@@ -21,17 +23,45 @@ var (
 	// in the migration.
 	migrateExcludeRefs []string
 
+	// migrateYes indicates that an answer of 'yes' should be presumed
+	// whenever 'git lfs migrate' asks for user input.
+	migrateYes bool
+
+	// migrateSkipFetch assumes that the client has the latest copy of
+	// remote references, and thus should not contact the remote for a set
+	// of updated references.
+	migrateSkipFetch bool
+
 	// migrateEverything indicates the presence of the --everything flag,
 	// and instructs 'git lfs migrate' to migrate all local references.
 	migrateEverything bool
 
 	// migrateVerbose enables verbose logging
 	migrateVerbose bool
+
+	// objectMapFile is the path to the map of old sha1 to new sha1
+	// commits
+	objectMapFilePath string
+
+	// migrateNoRewrite is the flag indicating whether or not the
+	// command should rewrite git history
+	migrateNoRewrite bool
+	// migrateCommitMessage is the message to use with the commit generated
+	// by the migrate command
+	migrateCommitMessage string
+
+	// exportRemote is the remote from which to download objects when
+	// performing an export
+	exportRemote string
+
+	// migrateFixup is the flag indicating whether or not to infer the
+	// included and excluded filepath patterns.
+	migrateFixup bool
 )
 
-// migrate takes the given command and arguments, *odb.ObjectDatabase, as well
+// migrate takes the given command and arguments, *gitobj.ObjectDatabase, as well
 // as a BlobRewriteFn to apply, and performs a migration.
-func migrate(args []string, r *githistory.Rewriter, l *log.Logger, opts *githistory.RewriteOptions) {
+func migrate(args []string, r *githistory.Rewriter, l *tasklog.Logger, opts *githistory.RewriteOptions) {
 	requireInRepo()
 
 	opts, err := rewriteOptions(args, opts, l)
@@ -47,12 +77,12 @@ func migrate(args []string, r *githistory.Rewriter, l *log.Logger, opts *githist
 
 // getObjectDatabase creates a *git.ObjectDatabase from the filesystem pointed
 // at the .git directory of the currently checked-out repository.
-func getObjectDatabase() (*odb.ObjectDatabase, error) {
+func getObjectDatabase() (*gitobj.ObjectDatabase, error) {
 	dir, err := git.GitDir()
 	if err != nil {
 		return nil, errors.Wrap(err, "cannot open root")
 	}
-	return odb.FromFilesystem(filepath.Join(dir, "objects"))
+	return gitobj.FromFilesystem(filepath.Join(dir, "objects"), cfg.TempDir())
 }
 
 // rewriteOptions returns *githistory.RewriteOptions able to be passed to a
@@ -68,7 +98,7 @@ func getObjectDatabase() (*odb.ObjectDatabase, error) {
 //
 // If any of the above could not be determined without error, that error will be
 // returned immediately.
-func rewriteOptions(args []string, opts *githistory.RewriteOptions, l *log.Logger) (*githistory.RewriteOptions, error) {
+func rewriteOptions(args []string, opts *githistory.RewriteOptions, l *tasklog.Logger) (*githistory.RewriteOptions, error) {
 	include, exclude, err := includeExcludeRefs(l, args)
 	if err != nil {
 		return nil, err
@@ -78,11 +108,13 @@ func rewriteOptions(args []string, opts *githistory.RewriteOptions, l *log.Logge
 		Include: include,
 		Exclude: exclude,
 
-		UpdateRefs: opts.UpdateRefs,
-		Verbose:    opts.Verbose,
+		UpdateRefs:        opts.UpdateRefs,
+		Verbose:           opts.Verbose,
+		ObjectMapFilePath: opts.ObjectMapFilePath,
 
-		BlobFn:         opts.BlobFn,
-		TreeCallbackFn: opts.TreeCallbackFn,
+		BlobFn:            opts.BlobFn,
+		TreePreCallbackFn: opts.TreePreCallbackFn,
+		TreeCallbackFn:    opts.TreeCallbackFn,
 	}, nil
 }
 
@@ -97,7 +129,7 @@ func rewriteOptions(args []string, opts *githistory.RewriteOptions, l *log.Logge
 //     arguments and the --include-ref= or --exclude-ref= flag(s) aren't given.
 //   - Include all references given in --include-ref=<ref>.
 //   - Exclude all references given in --exclude-ref=<ref>.
-func includeExcludeRefs(l *log.Logger, args []string) (include, exclude []string, err error) {
+func includeExcludeRefs(l *tasklog.Logger, args []string) (include, exclude []string, err error) {
 	hardcore := len(migrateIncludeRefs) > 0 || len(migrateExcludeRefs) > 0
 
 	if len(args) == 0 && !hardcore && !migrateEverything {
@@ -116,6 +148,12 @@ func includeExcludeRefs(l *log.Logger, args []string) (include, exclude []string
 	}
 
 	for _, name := range args {
+		var excluded bool
+		if strings.HasPrefix("^", name) {
+			name = name[1:]
+			excluded = true
+		}
+
 		// Then, loop through each branch given, resolve that reference,
 		// and include it.
 		ref, err := git.ResolveRef(name)
@@ -123,7 +161,11 @@ func includeExcludeRefs(l *log.Logger, args []string) (include, exclude []string
 			return nil, nil, err
 		}
 
-		include = append(include, ref.Name)
+		if excluded {
+			exclude = append(exclude, ref.Refspec())
+		} else {
+			include = append(include, ref.Refspec())
+		}
 	}
 
 	if hardcore {
@@ -137,25 +179,52 @@ func includeExcludeRefs(l *log.Logger, args []string) (include, exclude []string
 		include = append(include, migrateIncludeRefs...)
 		exclude = append(exclude, migrateExcludeRefs...)
 	} else if migrateEverything {
-		localRefs, err := git.LocalRefs()
+		refs, err := git.AllRefsIn("")
 		if err != nil {
 			return nil, nil, err
 		}
 
-		for _, ref := range localRefs {
-			include = append(include, ref.Name)
+		for _, ref := range refs {
+			switch ref.Type {
+			case git.RefTypeLocalBranch, git.RefTypeLocalTag,
+				git.RefTypeRemoteBranch, git.RefTypeRemoteTag:
+
+				include = append(include, ref.Refspec())
+			case git.RefTypeOther:
+				parts := strings.SplitN(ref.Refspec(), "/", 3)
+				if len(parts) < 2 {
+					continue
+				}
+
+				switch parts[1] {
+				// The following are GitLab-, GitHub-, VSTS-,
+				// and BitBucket-specific reference naming
+				// conventions.
+				case "merge-requests", "pull", "pull-requests":
+					include = append(include, ref.Refspec())
+				}
+			}
 		}
 	} else {
-		// Otherwise, if neither --include-ref=<ref> or
-		// --exclude-ref=<ref> were given, include no additional
-		// references, and exclude all remote references that are remote
-		// branches or remote tags.
-		remoteRefs, err := getRemoteRefs(l)
+		bare, err := git.IsBare()
 		if err != nil {
-			return nil, nil, err
+			return nil, nil, errors.Wrap(err, "fatal: unable to determine bareness")
 		}
 
-		exclude = append(exclude, remoteRefs...)
+		if !bare {
+			// Otherwise, if neither --include-ref=<ref> or
+			// --exclude-ref=<ref> were given, include no additional
+			// references, and exclude all remote references that
+			// are remote branches or remote tags.
+			remoteRefs, err := getRemoteRefs(l)
+			if err != nil {
+				return nil, nil, err
+			}
+
+			for _, rr := range remoteRefs {
+				exclude = append(exclude, rr.Refspec())
+			}
+		}
 	}
 
 	return include, exclude, nil
@@ -164,28 +233,40 @@ func includeExcludeRefs(l *log.Logger, args []string) (include, exclude []string
 // getRemoteRefs returns a fully qualified set of references belonging to all
 // remotes known by the currently checked-out repository, or an error if those
 // references could not be determined.
-func getRemoteRefs(l *log.Logger) ([]string, error) {
-	var refs []string
+func getRemoteRefs(l *tasklog.Logger) ([]*git.Ref, error) {
+	var refs []*git.Ref
 
 	remotes, err := git.RemoteList()
 	if err != nil {
 		return nil, err
 	}
 
-	w := l.Waiter("migrate: Fetching remote refs")
-	if err := git.Fetch(remotes...); err != nil {
-		return nil, err
+	if !migrateSkipFetch {
+		w := l.Waiter("migrate: Fetching remote refs")
+		if err := git.Fetch(remotes...); err != nil {
+			return nil, err
+		}
+		w.Complete()
 	}
-	w.Complete()
 
 	for _, remote := range remotes {
-		refsForRemote, err := git.RemoteRefs(remote)
+		var refsForRemote []*git.Ref
+		if migrateSkipFetch {
+			refsForRemote, err = git.CachedRemoteRefs(remote)
+		} else {
+			refsForRemote, err = git.RemoteRefs(remote)
+		}
+
 		if err != nil {
 			return nil, err
 		}
 
-		for _, ref := range refsForRemote {
-			refs = append(refs, formatRefName(ref, remote))
+		for _, rr := range refsForRemote {
+			// HACK(@ttaylorr): add remote name to fully-qualify
+			// references:
+			rr.Name = fmt.Sprintf("%s/%s", remote, rr.Name)
+
+			refs = append(refs, rr)
 		}
 	}
 
@@ -229,7 +310,7 @@ func currentRefToMigrate() (*git.Ref, error) {
 
 // getHistoryRewriter returns a history rewriter that includes the filepath
 // filter given by the --include and --exclude arguments.
-func getHistoryRewriter(cmd *cobra.Command, db *odb.ObjectDatabase, l *log.Logger) *githistory.Rewriter {
+func getHistoryRewriter(cmd *cobra.Command, db *gitobj.ObjectDatabase, l *tasklog.Logger) *githistory.Rewriter {
 	include, exclude := getIncludeExcludeArgs(cmd)
 	filter := buildFilepathFilter(cfg, include, exclude)
 
@@ -237,6 +318,56 @@ func getHistoryRewriter(cmd *cobra.Command, db *odb.ObjectDatabase, l *log.Logge
 		githistory.WithFilter(filter), githistory.WithLogger(l))
 }
 
+func ensureWorkingCopyClean(in io.Reader, out io.Writer) {
+	dirty, err := git.IsWorkingCopyDirty()
+	if err != nil {
+		ExitWithError(errors.Wrap(err,
+			"fatal: could not determine if working copy is dirty"))
+	}
+
+	if !dirty {
+		return
+	}
+
+	var proceed bool
+	if migrateYes {
+		proceed = true
+	} else {
+		answer := bufio.NewReader(in)
+	L:
+		for {
+			fmt.Fprintf(out, "migrate: override changes in your working copy? [Y/n] ")
+			s, err := answer.ReadString('\n')
+			if err != nil {
+				if err == io.EOF {
+					break L
+				}
+				ExitWithError(errors.Wrap(err,
+					"fatal: could not read answer"))
+			}
+
+			switch strings.TrimSpace(s) {
+			case "n", "N":
+				proceed = false
+				break L
+			case "y", "Y":
+				proceed = true
+				break L
+			}
+
+			if !strings.HasSuffix(s, "\n") {
+				fmt.Fprintf(out, "\n")
+			}
+		}
+	}
+
+	if proceed {
+		fmt.Fprintf(out, "migrate: changes in your working copy will be overridden ...\n")
+	} else {
+		Exit("migrate: working copy must not be dirty")
+	}
+}
+
 func init() {
 	info := NewCommand("info", migrateInfoCommand)
 	info.Flags().IntVar(&migrateInfoTopN, "top", 5, "--top=<n>")
@@ -245,6 +376,15 @@ func init() {
 
 	importCmd := NewCommand("import", migrateImportCommand)
 	importCmd.Flags().BoolVar(&migrateVerbose, "verbose", false, "Verbose logging")
+	importCmd.Flags().StringVar(&objectMapFilePath, "object-map", "", "Object map file")
+	importCmd.Flags().BoolVar(&migrateNoRewrite, "no-rewrite", false, "Add new history without rewriting previous")
+	importCmd.Flags().StringVarP(&migrateCommitMessage, "message", "m", "", "With --no-rewrite, an optional commit message")
+	importCmd.Flags().BoolVar(&migrateFixup, "fixup", false, "Infer filepaths based on .gitattributes")
+
+	exportCmd := NewCommand("export", migrateExportCommand)
+	exportCmd.Flags().BoolVar(&migrateVerbose, "verbose", false, "Verbose logging")
+	exportCmd.Flags().StringVar(&objectMapFilePath, "object-map", "", "Object map file")
+	exportCmd.Flags().StringVar(&exportRemote, "remote", "", "Remote from which to download objects")
 
 	RegisterCommand("migrate", nil, func(cmd *cobra.Command) {
 		cmd.PersistentFlags().StringVarP(&includeArg, "include", "I", "", "Include a list of paths")
@@ -253,18 +393,10 @@ func init() {
 		cmd.PersistentFlags().StringSliceVar(&migrateIncludeRefs, "include-ref", nil, "An explicit list of refs to include")
 		cmd.PersistentFlags().StringSliceVar(&migrateExcludeRefs, "exclude-ref", nil, "An explicit list of refs to exclude")
 		cmd.PersistentFlags().BoolVar(&migrateEverything, "everything", false, "Migrate all local references")
+		cmd.PersistentFlags().BoolVar(&migrateSkipFetch, "skip-fetch", false, "Assume up-to-date remote references.")
 
-		cmd.PersistentPreRun = func(_ *cobra.Command, args []string) {
-			// Initialize local storage before running any child
-			// subcommands, since migrations require lfs.TempDir to
-			// be initialized within ".git/lfs/objects".
-			//
-			// When lfs.TempDir is initialized to "/tmp",
-			// hard-linking can fail when another filesystem is
-			// mounted at "/tmp" (such as tmpfs).
-			localstorage.InitStorageOrFail()
-		}
+		cmd.PersistentFlags().BoolVarP(&migrateYes, "yes", "y", false, "Don't prompt for answers.")
 
-		cmd.AddCommand(importCmd, info)
+		cmd.AddCommand(exportCmd, importCmd, info)
 	})
 }
