@@ -5,89 +5,45 @@ package lfs
 import (
 	"fmt"
 	"os"
-	"path/filepath"
 	"sort"
 	"strings"
 
 	"github.com/git-lfs/git-lfs/config"
 	"github.com/git-lfs/git-lfs/lfsapi"
-	"github.com/git-lfs/git-lfs/localstorage"
 	"github.com/git-lfs/git-lfs/tools"
 	"github.com/git-lfs/git-lfs/tq"
 	"github.com/rubyist/tracerx"
 )
 
-// LocalMediaDir returns the root of lfs objects
-func LocalMediaDir() string {
-	if localstorage.Objects() != nil {
-		return localstorage.Objects().RootDir
-	}
-	return ""
-}
-
-func LocalObjectTempDir() string {
-	if localstorage.Objects() != nil {
-		return localstorage.Objects().TempDir
-	}
-	return ""
-}
-
-func TempDir() string {
-	return localstorage.TempDir
-}
-
-func TempFile(prefix string) (*os.File, error) {
-	return localstorage.TempFile(prefix)
-}
-
-func LocalMediaPath(oid string) (string, error) {
-	return localstorage.Objects().BuildObjectPath(oid)
-}
-
-func LocalMediaPathReadOnly(oid string) string {
-	return localstorage.Objects().ObjectPath(oid)
-}
-
-func LocalReferencePath(sha string) string {
-	if config.LocalReferenceDir == "" {
-		return ""
-	}
-	return filepath.Join(config.LocalReferenceDir, sha[0:2], sha[2:4], sha)
-}
-
-func ObjectExistsOfSize(oid string, size int64) bool {
-	path := localstorage.Objects().ObjectPath(oid)
-	return tools.FileExistsOfSize(path, size)
-}
-
 func Environ(cfg *config.Configuration, manifest *tq.Manifest) []string {
 	osEnviron := os.Environ()
 	env := make([]string, 0, len(osEnviron)+7)
 
-	api, err := lfsapi.NewClient(cfg.Os, cfg.Git)
+	api, err := lfsapi.NewClient(cfg)
 	if err != nil {
 		// TODO(@ttaylorr): don't panic
 		panic(err.Error())
 	}
 
-	download := api.Endpoints.AccessFor(api.Endpoints.Endpoint("download", cfg.CurrentRemote).Url)
-	upload := api.Endpoints.AccessFor(api.Endpoints.Endpoint("upload", cfg.CurrentRemote).Url)
+	download := api.Endpoints.AccessFor(api.Endpoints.Endpoint("download", cfg.Remote()).Url)
+	upload := api.Endpoints.AccessFor(api.Endpoints.Endpoint("upload", cfg.PushRemote()).Url)
 
 	dltransfers := manifest.GetDownloadAdapterNames()
 	sort.Strings(dltransfers)
 	ultransfers := manifest.GetUploadAdapterNames()
 	sort.Strings(ultransfers)
 
-	fetchPruneConfig := cfg.FetchPruneConfig()
-	storageConfig := cfg.StorageConfig()
+	fetchPruneConfig := NewFetchPruneConfig(cfg.Git)
+
+	references := strings.Join(cfg.LocalReferenceDirs(), ", ")
 
 	env = append(env,
-		fmt.Sprintf("LocalWorkingDir=%s", config.LocalWorkingDir),
-		fmt.Sprintf("LocalGitDir=%s", config.LocalGitDir),
-		fmt.Sprintf("LocalGitStorageDir=%s", config.LocalGitStorageDir),
-		fmt.Sprintf("LocalMediaDir=%s", LocalMediaDir()),
-		fmt.Sprintf("LocalReferenceDir=%s", config.LocalReferenceDir),
-		fmt.Sprintf("TempDir=%s", TempDir()),
+		fmt.Sprintf("LocalWorkingDir=%s", cfg.LocalWorkingDir()),
+		fmt.Sprintf("LocalGitDir=%s", cfg.LocalGitDir()),
+		fmt.Sprintf("LocalGitStorageDir=%s", cfg.LocalGitStorageDir()),
+		fmt.Sprintf("LocalMediaDir=%s", cfg.LFSObjectDir()),
+		fmt.Sprintf("LocalReferenceDirs=%s", references),
+		fmt.Sprintf("TempDir=%s", cfg.TempDir()),
 		fmt.Sprintf("ConcurrentTransfers=%d", api.ConcurrentTransfers),
 		fmt.Sprintf("TusTransfers=%v", cfg.TusTransfersAllowed()),
 		fmt.Sprintf("BasicTransfersOnly=%v", cfg.BasicTransfersOnly()),
@@ -99,7 +55,7 @@ func Environ(cfg *config.Configuration, manifest *tq.Manifest) []string {
 		fmt.Sprintf("PruneOffsetDays=%d", fetchPruneConfig.PruneOffsetDays),
 		fmt.Sprintf("PruneVerifyRemoteAlways=%v", fetchPruneConfig.PruneVerifyRemoteAlways),
 		fmt.Sprintf("PruneRemoteName=%s", fetchPruneConfig.PruneRemoteName),
-		fmt.Sprintf("LfsStorageDir=%s", storageConfig.LfsStorageDir),
+		fmt.Sprintf("LfsStorageDir=%s", cfg.LFSStorageDir()),
 		fmt.Sprintf("AccessDownload=%s", download),
 		fmt.Sprintf("AccessUpload=%s", upload),
 		fmt.Sprintf("DownloadTransfers=%s", strings.Join(dltransfers, ",")),
@@ -125,21 +81,6 @@ func Environ(cfg *config.Configuration, manifest *tq.Manifest) []string {
 	return env
 }
 
-func InRepo() bool {
-	return config.LocalGitDir != ""
-}
-
-func ClearTempObjects() error {
-	if localstorage.Objects() == nil {
-		return nil
-	}
-	return localstorage.Objects().ClearTempObjects()
-}
-
-func ScanObjectsChan() <-chan localstorage.Object {
-	return localstorage.Objects().ScanObjectsChan()
-}
-
 func init() {
 	tracerx.DefaultKey = "GIT"
 	tracerx.Prefix = "trace git-lfs: "
@@ -155,22 +96,23 @@ const (
 	gitPtrPrefix = "gitdir: "
 )
 
-// only used in tests
-func AllObjects() []localstorage.Object {
-	return localstorage.Objects().AllObjects()
-}
-
-func LinkOrCopyFromReference(oid string, size int64) error {
-	if ObjectExistsOfSize(oid, size) {
+func LinkOrCopyFromReference(cfg *config.Configuration, oid string, size int64) error {
+	if cfg.LFSObjectExists(oid, size) {
 		return nil
 	}
-	altMediafile := LocalReferencePath(oid)
-	mediafile, err := LocalMediaPath(oid)
+	altMediafiles := cfg.Filesystem().ObjectReferencePaths(oid)
+	mediafile, err := cfg.Filesystem().ObjectPath(oid)
 	if err != nil {
 		return err
 	}
-	if altMediafile != "" && tools.FileExistsOfSize(altMediafile, size) {
-		return LinkOrCopy(altMediafile, mediafile)
+	for _, altMediafile := range altMediafiles {
+		tracerx.Printf("altMediafile: %s", altMediafile)
+		if altMediafile != "" && tools.FileExistsOfSize(altMediafile, size) {
+			err = LinkOrCopy(cfg, altMediafile, mediafile)
+			if err == nil {
+				break
+			}
+		}
 	}
-	return nil
+	return err
 }
