@@ -7,6 +7,7 @@ import (
 	"strings"
 	"sync"
 
+	"github.com/git-lfs/git-lfs/fs"
 	"github.com/git-lfs/git-lfs/lfsapi"
 	"github.com/rubyist/tracerx"
 )
@@ -15,6 +16,7 @@ import (
 // process transfers with N workers handling an oid each, and which wait for
 // authentication to succeed on one worker before proceeding
 type adapterBase struct {
+	fs           *fs.Filesystem
 	name         string
 	direction    Direction
 	transferImpl transferImplementation
@@ -49,8 +51,9 @@ type transferImplementation interface {
 	DoTransfer(ctx interface{}, t *Transfer, cb ProgressCallback, authOkFunc func()) error
 }
 
-func newAdapterBase(name string, dir Direction, ti transferImplementation) *adapterBase {
+func newAdapterBase(f *fs.Filesystem, name string, dir Direction, ti transferImplementation) *adapterBase {
 	return &adapterBase{
+		fs:           f,
 		name:         name,
 		direction:    dir,
 		transferImpl: ti,
