@@ -1,13 +1,15 @@
 package tq
 
 import (
+	"fmt"
 	"os"
 	"sort"
 	"sync"
 
 	"github.com/git-lfs/git-lfs/errors"
+	"github.com/git-lfs/git-lfs/git"
 	"github.com/git-lfs/git-lfs/lfsapi"
-	"github.com/git-lfs/git-lfs/progress"
+	"github.com/git-lfs/git-lfs/tools"
 	"github.com/rubyist/tracerx"
 )
 
@@ -103,12 +105,13 @@ type TransferQueue struct {
 	direction         Direction
 	client            *tqClient
 	remote            string
+	ref               *git.Ref
 	adapter           Adapter
 	adapterInProgress bool
 	adapterInitMutex  sync.Mutex
 	dryRun            bool
-	cb                progress.CopyCallback
-	meter             progress.Meter
+	cb                tools.CopyCallback
+	meter             *Meter
 	errors            []error
 	transfers         map[string]*objects
 	batchSize         int
@@ -125,6 +128,11 @@ type TransferQueue struct {
 	wait     sync.WaitGroup
 	manifest *Manifest
 	rc       *retryCounter
+
+	// unsupportedContentType indicates whether the transfer queue ever saw
+	// an HTTP 422 response indicating that their upload destination does
+	// not support Content-Type detection.
+	unsupportedContentType bool
 }
 
 // objects holds a set of objects.
@@ -177,13 +185,19 @@ func DryRun(dryRun bool) Option {
 	}
 }
 
-func WithProgress(m progress.Meter) Option {
+func WithProgress(m *Meter) Option {
 	return func(tq *TransferQueue) {
 		tq.meter = m
 	}
 }
 
-func WithProgressCallback(cb progress.CopyCallback) Option {
+func RemoteRef(ref *git.Ref) Option {
+	return func(tq *TransferQueue) {
+		tq.ref = ref
+	}
+}
+
+func WithProgressCallback(cb tools.CopyCallback) Option {
 	return func(tq *TransferQueue) {
 		tq.cb = cb
 	}
@@ -223,13 +237,11 @@ func NewTransferQueue(dir Direction, manifest *Manifest, remote string, options
 	if q.bufferDepth <= 0 {
 		q.bufferDepth = q.batchSize
 	}
-
-	q.incoming = make(chan *objectTuple, q.bufferDepth)
-
-	if q.meter == nil {
-		q.meter = progress.Noop()
+	if q.meter != nil {
+		q.meter.Direction = q.direction
 	}
 
+	q.incoming = make(chan *objectTuple, q.bufferDepth)
 	q.collectorWait.Add(1)
 	q.errorwait.Add(1)
 	q.run()
@@ -423,7 +435,7 @@ func (q *TransferQueue) enqueueAndCollectRetriesFor(batch batch) (batch, error)
 		// Query the Git LFS server for what transfer method to use and
 		// details such as URLs, authentication, etc.
 		var err error
-		bRes, err = Batch(q.manifest, q.direction, q.remote, batch.ToTransfers())
+		bRes, err = Batch(q.manifest, q.direction, q.remote, q.ref, batch.ToTransfers())
 		if err != nil {
 			// If there was an error making the batch API call, mark all of
 			// the objects for retry, and return them along with the error
@@ -645,8 +657,13 @@ func (q *TransferQueue) handleTransferResult(
 			// If the error wasn't retriable, OR the object has
 			// exceeded its retry budget, it will be NOT be sent to
 			// the retry channel, and the error will be reported
-			// immediately.
-			q.errorc <- res.Error
+			// immediately (unless the error is in response to a
+			// HTTP 422).
+			if errors.IsUnprocessableEntityError(res.Error) {
+				q.unsupportedContentType = true
+			} else {
+				q.errorc <- res.Error
+			}
 			q.wait.Done()
 		}
 	} else {
@@ -757,6 +774,17 @@ func (q *TransferQueue) toAdapterCfg(e lfsapi.Endpoint) AdapterConfig {
 	}
 }
 
+var (
+	// contentTypeWarning is the message printed when a server returns an
+	// HTTP 422 at the end of a push.
+	contentTypeWarning = []string{
+		"Uploading failed due to unsupported Content-Type header(s).",
+		"Consider disabling Content-Type detection with:",
+		"",
+		"  $ git config lfs.contenttype false",
+	}
+)
+
 // Wait waits for the queue to finish processing all transfers. Once Wait is
 // called, Add will no longer add transfers to the queue. Any failed
 // transfers will be automatically retried once.
@@ -773,8 +801,14 @@ func (q *TransferQueue) Wait() {
 		close(watcher)
 	}
 
-	q.meter.Finish()
+	q.meter.Flush()
 	q.errorwait.Wait()
+
+	if q.unsupportedContentType {
+		for _, line := range contentTypeWarning {
+			fmt.Fprintf(os.Stderr, "info: %s\n", line)
+		}
+	}
 }
 
 // Watch returns a channel where the queue will write the value of each transfer
